import { For, Show, createEffect, createMemo, createSignal } from "solid-js";
import { DownloadIcon } from "../../icons/DownloadIcon";
import { Person } from "../../types/Person";
import { Register } from "../../types/Register";
import { courseMap } from "../../utils/allCourses";
import { certGenerator } from "../../certGenerator";
import { CaretRight } from "../../icons/CaretRight";
import { RegisterSidebar } from "./RegisterSidebar";

export function Registers(props: {person: Person | null, count: number}) {
    const [registers, setRegisters] = createSignal<Array<Register>>([]);
    const [showHidden, setShowHidden] = createSignal(false);
    const [sidebarRegister, setSidebarRegister] = createSignal<Register | null>(null);

    const loadRegisters = async() => {
        const person = props.person!;

        const res = await fetch(`${import.meta.env.VITE_BACKEND_URL}/api/register/${person.person_dni}`);
        if (res.ok) {
            const data = await res.json();
            setRegisters(data);
        }
    };

    createEffect(async() => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const _count = props.count;

        if (props.person === null) {
            setRegisters([]);
            return;
        }

        loadRegisters();
    });

    // All registers sorted by year, and an indicator of whether there's register older
    // than last year
    const registerSortedList = createMemo<[Array<Array<Register>>, boolean]>(() => {
        const allRegisters = registers();

        const registerByYear: {[y: string]: Array<Register>} = {};

        let olderThanLastYear = false;
        allRegisters.forEach((register) => {
            const year = new Date(register.register_display_date).getFullYear();
            const currentYear = new Date().getFullYear();

            // Don't show registers from before last year
            if (year < (currentYear - 1)) {
                olderThanLastYear = true;
                if (!showHidden()) {
                    return;
                }
            }

            if (registerByYear[year] === undefined) {
                registerByYear[year] = [];
            }

            registerByYear[year].push(register);
        });

        // Create a new array with each year's registers
        // Sort the object keys (years) in descending order
        const sortedYears = Object.keys(registerByYear).sort((y1, y2) => (y1 < y2 ? 1 : -1));

        // Create the final array
        const result = sortedYears.map((year) => registerByYear[year].sort((r1, r2) => (r1.register_display_date < r2.register_display_date ? 1 : -1)));

        return [result, olderThanLastYear];
    });

    const inputCheck = () => setShowHidden((x) => !x);

    return (
        <div class="m-4 p-4 rounded-md relative">
            <h3 class="text-xl font-medium">
                {props.person?.person_names}&nbsp;
                {props.person?.person_paternal_surname}&nbsp;
                {props.person?.person_maternal_surname}

                <Show when={registerSortedList()[1]}>
                    &nbsp;&nbsp;&nbsp;|
                    <input id={`person_${props.person?.person_id ?? "_"}`} type="checkbox" class="ml-4 mr-2" checked oninput={inputCheck} />
                    <label for={`person_${props.person?.person_id ?? "_"}`} class="text-sm">
                        Ocultar cert. anteriores a {new Date().getFullYear() - 1}
                    </label>
                </Show>
            </h3>

            <For each={registerSortedList()[0]}>
                {(year) => (
                    <div class="flex flex-wrap justify-start gap-2 my-6">
                        <For each={year}>
                            {(register) => (
                                <RegisterEl
                                    register={register}
                                    person={props.person!}
                                    onClick={() => setSidebarRegister(register)}
                                />
                            )}
                        </For>
                    </div>
                )}
            </For>

            <Show when={sidebarRegister() !== null}>
                <RegisterSidebar
                    register={sidebarRegister()!}
                    close={() => setSidebarRegister(null)}
                    onDelete={loadRegisters}
                />
            </Show>
        </div>
    );
}

function RegisterEl(props: {register: Register, person: Person, onClick: () => void}) {

    const dateComponents = () => {
        const [, year, month, day] = /(\d{4})-(\d{2})-(\d{2})/.exec(props.register.register_display_date) ?? [];
        return {year, month, day};
    };

    const displayDate = () => {
        const {year, month, day} = dateComponents();
        return `${day}/${month}/${year}`;
    };

    const courseName = () => {
        const course = courseMap()[props.register.register_course_id];
        return course?.course_name ?? "Curso no encontrado";
    };

    const genCert = () => {
        const courseN = courseName();
        const person = props.person;
        const register = props.register;

        const generator = certGenerator[courseN];
        if (generator === undefined) {
            console.error(`Generator function with key \`${courseN}\` not found`);
            return;
        }

        const {year, month, day} = dateComponents();
        const personFullName = `${person.person_names} ${person.person_paternal_surname} ${person.person_maternal_surname}`;

        generator(`${courseN} - ${personFullName}.docx`, {
            matpel: null,
            personFullName,
            personDni: person.person_dni.toString(),
            certCode: register.register_code.toString().padStart(4, "0"),
            certYear: year,
            certMonth: month,
            certDay: day,
            certIId: register.register_code,
        });
    };

    const generateable = () => {
        const courseN = courseName();
        return certGenerator[courseN] !== undefined;
    };

    return (
        <div class="grid grid-cols-[3rem_9rem_1.25rem] w-[13.75rem] p-1 border border-c-outline rounded-md">
            <div class="h-12 w-12 inline-block">
                <Show when={generateable()}>
                    <button
                        onclick={genCert}
                        class="rounded-full bg-c-primary-container hover:bg-c-primary transition-colors h-full w-full"
                    >
                        <DownloadIcon fill="var(--c-on-primary-container)" />
                    </button>
                </Show>
            </div>
            <div class="h-12 pl-2 align-middle">
                <p
                    class="font-bold overflow-hidden whitespace-nowrap"
                    title={courseName()}
                >
                    {courseName()}
                </p>
                <p class="font-mono text-sm">
                    {displayDate()}
                    &nbsp;-&nbsp;
                    {props.register.register_code}
                </p>
            </div>
            <button
                class="hover:bg-c-surface-variant rounded-md transition-colors"
                title="Más opciones"
                onclick={props.onClick}
            >
                <CaretRight fill="var(--c-primary)" size={20} />
            </button>
        </div>
    );
}
